package cz.muni.fi.group05.room03.ui;

import cz.muni.fi.group05.room03.ui.form.GuestForm;
import cz.muni.fi.group05.room03.ui.form.ReservationForm;
import cz.muni.fi.group05.room03.ui.table.GuestTable;
import cz.muni.fi.group05.room03.ui.table.ReservationTable;
import cz.muni.fi.group05.room03.ui.table.RoomTable;

import javax.swing.JMenuBar;
import javax.swing.JPanel;
import java.awt.CardLayout;
import java.awt.GridLayout;

public class ContentPanelController {

    private static final I18N I18N = new I18N(ContentPanelController.class);
    private static final JPanel rootPanel = new JPanel(new CardLayout());
    private static final CardLayout cards = (CardLayout) rootPanel.getLayout();
    private static final ColoredButton reservationsButton = createMenuButton(I18N.getString("reservations"), ContentPanelNames.RESERVATION_TABLE);
    private static final ColoredButton guestsButton = createMenuButton(I18N.getString("guests"), ContentPanelNames.GUEST_TABLE);
    private static final ColoredButton roomsButton = createMenuButton(I18N.getString("rooms"), ContentPanelNames.ROOM_TABLE);

    private static ContentPanelNames currentActive = ContentPanelNames.RESERVATION_TABLE;

    private static ColoredButton createMenuButton(String name, ContentPanelNames pane) {
        var menu = ColoredButton.menu(name);
        menu.addActionListener(e -> ContentPanelController.switchPanel(pane));
        return menu;
    }

    public static void addPanels(ReservationTable reservationTable, GuestTable guestTable,
                                 RoomTable roomTable, ReservationForm reservationForm, GuestForm guestForm) {
        rootPanel.add(reservationTable.getPanel(), ContentPanelNames.RESERVATION_TABLE.name());
        rootPanel.add(guestTable.getPanel(), ContentPanelNames.GUEST_TABLE.name());
        rootPanel.add(roomTable.getPanel(), ContentPanelNames.ROOM_TABLE.name());
        rootPanel.add(reservationForm.getPanel(), ContentPanelNames.RESERVATION_FORM.name());
        rootPanel.add(guestForm.getPanel(), ContentPanelNames.GUEST_FORM.name());
    }

    private static void switchPanelMaybe(ContentPanelNames panelName, ContentPanelActions action) {
        boolean doSwitch = true;
        if((currentActive == ContentPanelNames.RESERVATION_FORM || currentActive == ContentPanelNames.GUEST_FORM)
                && (action == ContentPanelActions.CANCEL)) {
            doSwitch = Message.showYesNoDialog(I18N.getString("switchCheckTitle"),
                    I18N.getString("switchQuestionMessage") + '\n' + I18N.getString("switchWarningMessage"));
        }
        if (doSwitch) {
            cards.show(rootPanel, panelName.name());
            reservationsButton.setActive(panelName == ContentPanelNames.RESERVATION_TABLE || panelName == ContentPanelNames.RESERVATION_FORM);
            guestsButton.setActive(panelName == ContentPanelNames.GUEST_TABLE || panelName == ContentPanelNames.GUEST_FORM);
            roomsButton.setActive(panelName == ContentPanelNames.ROOM_TABLE);
            currentActive = panelName;
        }
    }

    public static void switchPanel(ContentPanelNames panelName){
        switchPanelMaybe(panelName, ContentPanelActions.CANCEL);
    }

    public static void switchPanelDialog(ContentPanelNames panelName) {
        switchPanelMaybe(panelName, ContentPanelActions.CONFIRM);
    }

    public static Runnable switchPanelAction(ContentPanelNames panelName) {
        return () -> switchPanel(panelName);
    }

    public static JMenuBar createButtonMenu() {
        var menuBar = new JMenuBar();
        menuBar.setLayout(new GridLayout(1, 3));
        menuBar.add(reservationsButton);
        menuBar.add(guestsButton);
        menuBar.add(roomsButton);
        return menuBar;
    }

    public static JPanel getPanel(){
        return rootPanel;
    }

    public enum ContentPanelNames {
        RESERVATION_TABLE,
        RESERVATION_FORM,
        GUEST_TABLE,
        GUEST_FORM,
        ROOM_TABLE
    }

    private enum ContentPanelActions {
        CONFIRM,
        CANCEL
    }
}
